<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
   NSG WebGUI

   Software distributed under the License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
   for the specific language governing rights and limitations under the
   License.

   Licence information can be found within the file LICENCE.php

   The Original Code is SMG WebGUI

   The Initial Developer of the Original Code is
   Shaunt Libarian <shaunt@sangoma.com>

   Contributor(s):
   Abhishek Arora <aarora@sangoma.com>
 */

require_once 'sangoma/database_classes.php';
class ss7_globalconfig extends database_class
{
    public $error= "";
    public $error_counter="";
    public $db_location = "sqlite:../sqlite/cardinfo.db";
    public $defaultValues = array();
    // Define table schema
    // Would have been nice to get an array but this is how most db functions expect schema in NSG
    private $table_name = "ss7_global_config";
    private $table_schema = 
        "parameter varchar(128), value varchar(256), PRIMARY KEY(parameter)";
    private $table_fields = 
        "parameter,value";
    private $table_column_count = 2;

    /* Global config fields definition */
    private $_fields_def = array(
        'hw_rj45_port_map' => array(
            'title' => 'HW RJ45 Port Map', 
            'category' => 'driver',
            'default' => 'default',
            'type' => 'enum',
            'options' => array('default'=>'Default', 'linear'=>'Linear'),
            'rules' => '',
            'help' => '(Applicable to A108 cards only)',
        ),
        'sip-headers' => array(
            'title' => 'SIP X-Headers', 
            'category' => 'sip',
            'default' => 'enable',
            'type' => 'enum',
            'options' => array('disable'=>'Disable', 'enable'=>'Enable'),
            'rules' => '',
            'help' => '',
        ),
        'force_inr' => array(
            'title' => 'ISUP INR', 
            'category' => 'isup',
            'default' => 'disable',
            'type' => 'enum',
            'options' => array('disable'=>'Disable', 'enable'=>'Enable'),
            'rules' => '',
            'help' => '',
        ),
        'set-isup-message-priority' => array(
            'title' => 'ISUP User Message Priority', 
            'category' => 'isup',
            'default' => 'default',
            'type' => 'enum',
            'options' => array('default'=>'Default', 'all-ones'=>'All-ones', 'all-zeros'=>'All-zeros'),
            'rules' => '',
            'help' => '',
            // Backward compatibility remapping
            // old value => new value
            'remap' => array(
                '7' => 'default',
                '0' => 'all-zeros',
                '87381' => 'all-ones',
            ),
        ),
        'auto-congestion-control' => array(
            'title' => 'Auto Congestion', 
            'category' => 'acc',
            'default' => 'disable',
            'type' => 'enum',
            'options' => array('disable'=>'Disable', 'enable'=>'Enable'),
            'rules' => '',
            'help' => '',
        ),
        'first-level-reduction-rate' => array(
            'title' => 'Call Control Rate at Congestion Level 1', 
            'category' => 'acc',
            'default' => 50,
            'type' => 'text',
            'rules' => '',
            'help' => '',
        ),
        'second-level-reduction-rate' => array(
            'title' => 'Call Control Rate at Congestion Level 2', 
            'category' => 'acc',
            'default' => 90,
            'type' => 'text',
            'rules' => '',
            'help' => '',
        ),
        'traffic-reduction-rate' => array(
            'title' => 'Traffic Reduction Rate', 
            'category' => 'acc',
            'default' => 10,
            'type' => 'text',
            'rules' => '',
            'help' => '',
        ),
        'traffic-increment-rate' => array(
            'title' => 'Traffic Increment Rate', 
            'category' => 'acc',
            'default' => 10,
            'type' => 'text',
            'rules' => '',
            'help' => '',
        ),
        'max-cpu-usage' => array(
            'title' => 'Max CPU Usage', 
            'category' => 'acc',
            'default' => 80,
            'type' => 'text',
            'rules' => '',
            'help' => '',
        ),

    );

    /**
     * @brief Initialize global config
     * make sure table exists and properly structured and populated
     *
     * @param[in out] $db_location
     *
     * @return 
     */
    public function initialize($db_location=null) {
        if(null == $db_location){
            $db_location = $this->db_location;
        }

        // Check table exists
        if(!$this->check_for_table($db_location, $this->table_name)){
            // Table doesn't exist, create
            $this->create_table($db_location, $this->table_name, $this->table_schema);
            $this->create_table($db_location, 'perm_'.$this->table_name, $this->table_schema);
        }else{
            // table exists, check schema
            $tables = array( $this->table_name, 'perm_'.$this->table_name);
            foreach($tables as $t){
                $this->check_schema(
                    $db_location, $t, $this->table_schema, 
                    (string)$this->table_column_count, $this->table_fields, $replaced); 
                // Remove old config entries
                // SQL= DELETE FROM ss7_global_config WHERE parameter NOT IN (<list of all existing records>);
                $_fields = array_keys($this->_fields_def);
                $_exising_entries = "'" . implode("','", $_fields) . "'";
                $this->delete_generic($db_location, $t,
                    "parameter NOT IN (" . $_exising_entries . ")");
            }
        }
        // Ensure all entries exist in table
        // + check for record value migration
        $_records = $this->get_field_records();
        // Loop all record definition
        foreach($this->_fields_def as $k => $v){
            // Record exists ?
            $value = $this->select_statement($db_location, $this->table_name, $this->table_fields, "parameter='".$k."'");
            if(empty($value)){
                // Record doesn't exists, insert
                $this->insert_statement($db_location, $this->table_name, $this->table_fields, $_records[$k]);
            }else{
                // Record exists
                // Migrate if needed (remap present in field_def and matching
                // value in remap array)
                if(isset($v['remap']) && array_key_exists($value['value'], $v['remap'])){
                    // Update record with remapped value
                    $this->update_table($db_location, $this->table_name,
                        "value='".$v['remap'][$value['value']]."'",
                        "parameter='".$k."'");

                }
            }
        }
    }
    /**
     * @brief Return a field definition
     *
     * @param[in out] $field_name
     *
     * @return 
     */
    public function get_field_def($field_name) {
        // TODO: Check field exists.
        return $this->_fields_def[$field_name];
    }

    /**
     * @brief Returns all field records for table insert
     *
     * array of:
     *  parameter name, parameter value
     *  TODO: Hack to comply to clumsy create table function
     *  [key] = "'key', 'value'"
     *
     * @return 
     */
    public function get_field_records() {
        $_records = array();
        foreach($this->_fields_def as $k => $d){
            $_records[$k] = "'$k' , '" . $d['default'] . "'";
        }
        return $_records;
    }

	function display()
	{
		?>
			<script type="text/javascript">
			$(document).ready(function(){

					$('#auto-congestion-control').change(function(){
						val = $(this).val();
						if(val=='disable')
						{
						$('.acc_textfields').attr('disabled', 'disabled');
						}
						else
						{
						$('.acc_textfields').attr('disabled', false);
						}
						});
					});
		</script>
			<form method="POST">
			<table class="mytable" width="100%" cellpadding='5'>
			<tbody><tr><th colspan="5" class="widget-content-header ui-state-active ui-corner-top" align="left">Driver</th>
            <?php foreach($this->defaultValues as $globalParameters) { if($globalParameters['category']=='driver'){ /*if($this->getcardname() !='A108') {$status = "disabled='disable'";}*/?>
				<tr>
					<td class="mytablesubheader" width="50%"><?php echo $globalParameters['title'];?></td>
                    <td width="50%" colspan="2" align="left"><?php echo $this->getFields($globalParameters['type'], $globalParameters['value'], $globalParameters['parameter'], $globalParameters['category'], $status, $globalParameters['options'])?> &nbsp; <i><?php echo($globalParameters['help']);?></i></td>
					</tr>
					<?php }} ?>
					</tbody>
					</table>
					<!--<div onclick="this.style.display='none';document.getElementById('hide_sip').style.display='block';document.getElementById('sipoptions').removeAttribute('style');
		ResizeFrame();" id="show_sip" class="fakelink" style="display: block; margin:10px"> >> Show Sip Options</div>
			<div onclick="this.style.display='none';document.getElementById('show_sip').style.display='block';document.getElementById('sipoptions').style.display='none';
		ResizeFrame();" id="hide_sip" class="fakelink" style="display: none;margin:10px"> >> Hide Sip Options</div>-->
			<br/>
			<table class="mytable" width="100%" cellpadding='5' id="sipoptions" <?php //if($this->error_counter==''){ echo "style='display:none;'";}?>>
			<tbody><tr><th colspan="5" class="widget-content-header ui-state-active ui-corner-top" align="left">SIP</th>
			<?php foreach($this->defaultValues as $globalParameters) { if($globalParameters['category']=='sip'){?>
				<tr>
					<td class="mytablesubheader" width="50%"><?php echo $globalParameters['title'];?></td>
					<td width="30%" align="left"><?php echo $this->getFields($globalParameters['type'], $globalParameters['value'], $globalParameters['parameter'], $globalParameters['category'], false, $globalParameters['options'])?></td>
					<td width="20%"><?php echo $this->error[$globalParameters['parameter']];?></td>
					</tr>
					<?php }} ?>
					</tbody>
					</table>
					<!--<div onclick="this.style.display='none';document.getElementById('hide_e').style.display='block';document.getElementById('advoptions').removeAttribute('style');
		ResizeFrame();" id="show_e" class="fakelink" style="display: block; margin:10px"> >>Show SS7 Options</div>
			<div onclick="this.style.display='none';document.getElementById('show_e').style.display='block';document.getElementById('advoptions').style.display='none';
		ResizeFrame();" id="hide_e" class="fakelink" style="display: none;margin:10px"> >>Hide SS7 Options</div>-->
			<br/>
			<table class="mytable" id="advoptions" width="100%" cellpadding='5' <?php //if($this->error_counter==''){ echo "style='display:none;'";}?>>
			<tbody>
			<tr><th colspan="5" class="widget-content-header ui-state-active ui-corner-top" align="left">ISUP</th>
			<?php foreach($this->defaultValues as $globalParameters) { if($globalParameters['category']=='isup'){?>
				<tr>
					<td class="mytablesubheader" width="50%"><?php echo $globalParameters['title'];?></td>
					<td width="30%" align="left"><?php echo $this->getFields($globalParameters['type'], $globalParameters['value'], $globalParameters['parameter'], $globalParameters['category'], $disable, $globalParameters['options'])?>
					</td>
					<td width="20%"><?php echo $this->error[$globalParameters['parameter']];?></td>
					</tr>
					<?php }} ?>
					<?php $disable=''; foreach($this->defaultValues as $globalParameters) { if($globalParameters['parameter'] == 'auto-congestion-control' && $globalParameters['value'] == 'disable'){ $disable = true; }} ?>
					<tr><th colspan="5" style = "background:#c3c4c5; color:#004581; border-bottom:1px solid;" align="left">Automatic Congestion Control</th>
					</tr>
					<?php foreach($this->defaultValues as $globalParameters) { if($globalParameters['category']=='acc'){?>
						<tr>
							<td class="mytablesubheader" width="50%"><?php echo $globalParameters['title'];?></td>
							<td width="30%" align="left"><?php echo $this->getFields($globalParameters['type'], $globalParameters['value'], $globalParameters['parameter'], $globalParameters['category'], $disable, $globalParameters['options'])?>
							</td>
							<td width="20%"><?php echo $this->error[$globalParameters['parameter']];?></td>
							</tr>
							<?php }} ?>
							</tbody></table>
							<br>
							<table width="100%" cellpadding='5'>
							<tbody><tr>
							<td width="1%"><input name="save_ss7globalconfig" value="Save Configuration" class="ui-state-default ui-corner-all" type="submit"></td>
							<td align="right"></td>
							</tr>
							</tbody></table>
							</form>
							<?php
	}
	public function validation($update=false)
	{
		if($update['auto-congestion-control'] == 'disable')
		{
			return true;
		}
		if($update['max-cpu-usage'] == '' || !is_numeric($update['max-cpu-usage']) || (intval($update['max-cpu-usage']) > 90) || (intval($update['max-cpu-usage']) < 10))
		{
			$this->error['max-cpu-usage'] = "<font color='red'>Max CPU must be numeric and between 10-90.</font>";
			$this->error_counter++;	
		}
		if($update['first-level-reduction-rate'] == '' || !is_numeric($update['first-level-reduction-rate']) || (intval($update['first-level-reduction-rate']) > 90) || (intval($update['first-level-reduction-rate']) < 1))
		{
			$this->error['first-level-reduction-rate'] = "<font color='red'>Call Control Rate at Congestion Level 1 must be numeric and between 1-90.</font>";
			$this->error_counter++;	
		}
		if($update['second-level-reduction-rate'] == '' || !is_numeric($update['second-level-reduction-rate']) || (intval($update['second-level-reduction-rate']) > 100) || (intval($update['second-level-reduction-rate']) < 1)  || (intval($update['first-level-reduction-rate']) > intval($update['second-level-reduction-rate'])))
		{
			$this->error['second-level-reduction-rate'] = "<font color='red'>Call Control Rate at Congestion Level 2 must be numeric, greater than Call Control Rate at Congestion Level 1 and between 1-100.</font>";
			$this->error_counter++;	
		}
		if($update['traffic-reduction-rate'] == '' || !is_numeric($update['traffic-reduction-rate']) || (intval($update['traffic-reduction-rate']) < 1) || (intval($update['traffic-reduction-rate']) > intval($update['first-level-reduction-rate'])))
		{
			$this->error['traffic-reduction-rate'] = "<font color='red'>Traffic reduction rate must be numeric, less than Call Control Rate at Congestion Level 1 and greater than 0.</font>";
			$this->error_counter++;	
		}	
		if($update['traffic-increment-rate'] == '' || !is_numeric($update['traffic-increment-rate']) || (intval($update['traffic-increment-rate']) < 1) ||  (intval($update['traffic-increment-rate']) > intval($update['first-level-reduction-rate'])))
		{
			$this->error['traffic-increment-rate'] = "<font color='red'>Traffic increment rate must be numeric, less than Call Control Rate at Congestion Level 1 and greater than 0.</font>";
			$this->error_counter++;	
		}
	
	
	}

    public function getDefaultValues($db_location=null)
    {
        if(empty($this->defaultValues)){
            // get records
            $values = $this->array_select_statement(
                ($db_location)?$db_location:$this->db_location, $this->table_name , '*');
            // Rework to adjust key/value pairs records to add field def
            // Iterate records: parameter, value
            foreach($values as $v) {
                // Get field definition
                $_def = $this->get_field_def($v['parameter']);
                // Update value in definition from record
                $_def['value'] = $v['value'];
                $_def['parameter'] = $v['parameter'];
                // Create records for view
                $this->defaultValues[] = $_def;
            }
        }
        return $this->defaultValues;
    }
	public function updateData($update)
	{
		unset($update['save_ss7globalconfig']);
		unset($update['PHPSESSID']);
		foreach($update as $key=>$saveData)
		{
			$this->update_table($this->db_location, $this->table_name, "value = '$saveData'", "parameter='$key'");
		}
		return true;
	}
    public function getFields($type,$value,$name,$class,$status,$options)
    {
        if($type=='text')
        {
            $disable = '';
            if($status == true){$disable = "disabled = 'disable'";}
            return "<input type='text' value='".$value."' name='".$name."' class = '".$class."_textfields' id='".$name."' style='text-align:left; width:80px' ".$disable.">";
        }
        if($type=='enum')
        {
            $select = "<select name='".$name."' id='".$name."' ".$status.">";
            $selected = '';
            foreach($options as $key=>$option){
                if($key == $value){
                    $selected = "selected='selected'";
                }else{
                    $selected = '';
                }
                $select.="<option value='".$key."' ".$selected.">".$option."</option>";
            }
            $select.="</select>";
            return $select;
        }
    }
	public function getcardname()
	{
		 $cardname = $this->array_select_statement($this->db_location, 'card_present' , 'dev_name');
		 if(!empty($cardname)){
			 $card =  explode('-', $cardname[0]['dev_name']);
			 return $card[1];
		 }
	}
}
?>
